#! /usr/bin/env python
# -*- coding: utf-8 -*-

SOUND_GENERAL_DOORBELL = 29
SOUND_GENERAL_ALAM = 30

SOUND_ROBOT_SPEED = 18
SOUND_ROBOT_TRANSFORM = 19
SOUND_ROBOT_SING = 20
SOUND_ROBOT_GUN = 21
SOUND_ROBOT_LASER = 22
SOUND_ROBOT_BROKEN = 23
SOUND_ROBOT_BOMB = 24

SOUND_DINOSAUR_ANGER = 25
SOUND_DINOSAUR_ATTACK = 26
SOUND_DINOSAUR_FIGHT = 27
SOUND_DINOSAUR_INJURED = 28


INSTRUMENTS_PIANO = 0
INSTRUMENTS_MUSIC_BOX = 10
INSTRUMENTS_BASS = 32
INSTRUMENTS_GUITAR = 24
INSTRUMENTS_SAXOPHONE = 64
INSTRUMENTS_FLUTE = 72
INSTRUMENTS_XYLOPHONE = 13
INSTRUMENTS_VIOLIN = 40
INSTRUMENTS_HARP = 46
INSTRUMENTS_PERCUSSION = 0

MIDI_DO_LOW_LOW = 36
MIDI_RE_LOW_LOW = 38
MIDI_MI_LOW_LOW = 40
MIDI_FA_LOW_LOW = 41
MIDI_SO_LOW_LOW = 43
MIDI_LA_LOW_LOW = 45
MIDI_SI_LOW_LOW = 47

MIDI_DO_LOW = 48
MIDI_RE_LOW = 50
MIDI_MI_LOW = 52
MIDI_FA_LOW = 53
MIDI_SO_LOW = 55
MIDI_LA_LOW = 57
MIDI_SI_LOW = 59

MIDI_DO = 60
MIDI_RE = 62
MIDI_MI = 64
MIDI_FA = 65
MIDI_SO = 67
MIDI_LA = 69
MIDI_SI = 71

MIDI_DO_HIGH = 72
MIDI_RE_HIGH = 74
MIDI_MI_HIGH = 76
MIDI_FA_HIGH = 77
MIDI_SO_HIGH = 79
MIDI_LA_HIGH = 81
MIDI_SI_HIGH = 83

MIDI_DO_HIGH_HIGH = 84
MIDI_RE_HIGH_HIGH = 86
MIDI_MI_HIGH_HIGH = 88
MIDI_FA_HIGH_HIGH = 89
MIDI_SO_HIGH_HIGH = 91
MIDI_LA_HIGH_HIGH = 93
MIDI_SI_HIGH_HIGH = 95

PERCUSSION_ACOUSTIC_BASS_DRUM = 35
PERCUSSION_BASS_DRUM_1 = 36
PERCUSSION_SIDE_STICK = 37
PERCUSSION_ACOUSTIC_SNARE = 38
PERCUSSION_HAND_CLAP = 39
PERCUSSION_ELECTRIC_SNARE = 40
PERCUSSION_LOW_FLOOR_TOM = 41
PERCUSSION_CLOSED_HI_HAT = 44
PERCUSSION_LOW_TOM = 45
PERCUSSION_OPEN_HI_HAT = 46
PERCUSSION_LOW_MID_TOM = 47
PERCUSSION_HI_MID_TOM = 48
PERCUSSION_CRASH_CYMBAL_1 = 49
PERCUSSION_HIGH_TOM = 50
PERCUSSION_RIDE_CYMBAL_1 = 51
PERCUSSION_CHINESE_CYMBAL = 52
PERCUSSION_RIDE_BELL = 53
PERCUSSION_TAMBOURINE = 54
PERCUSSION_SPLASH_CYMBAL = 55
PERCUSSION_COWBELL = 56
PERCUSSION_CRASH_CYMBAL_2 = 57
PERCUSSION_VIBRASLAP = 58
PERCUSSION_RIDE_CYMBAL_2 = 59
PERCUSSION_HI_BONGO = 60
PERCUSSION_LOW_BONGO = 61
PERCUSSION_MUTE_HI_CONGA = 62
PERCUSSION_OPEN_HI_CONGA = 63
PERCUSSION_LOW_CONGA = 64
PERCUSSION_HIGH_TIMBALE = 65
PERCUSSION_LOW_TIMBALE = 66
PERCUSSION_HIGH_AGOGO = 67
PERCUSSION_LOW_AGOGO = 68
PERCUSSION_CABASA = 69
PERCUSSION_MARACAS = 70
PERCUSSION_SHORT_WHISTLE = 71
PERCUSSION_LONG_WHISTLE = 72
PERCUSSION_SHORT_GUIRO = 73
PERCUSSION_LONG_GUIRO = 74
PERCUSSION_CLAVES = 75
PERCUSSION_HI_WOOD_BLOCK = 76
PERCUSSION_LOW_WOOD_BLOCK = 77
PERCUSSION_MUTE_GUICA = 78
PERCUSSION_OPEN_CUICA = 79
PERCUSSION_MUTE_TRIANGLE = 80
PERCUSSION_OPEN_TRIANGLE = 81



RECORD_START = 0x01
RECORD_END = 0x02
RECORD_PLAY = 0x03





MOTOR_PORTA = 0
MOTOR_PORTB = 1
MOTOR_PORTC = 2
MOTOR_PORTD = 3



SENSOR_DIGITAL_SERVO = 0x8B
SENSOR_OPTICAL = 0x52
SENSOR_BUZZER = 0x73
SENSOR_RGBLIGHT = 0x75
SENSOR_PHOTOSENSITIVE = 0x5B
SENSOR_LASER = 0x51
# SENSOR_FLAME = 0x55
SENSOR_GEOMAGNETIC = 0x56
SENSOR_ATTITUDE = 0x57

DIGITALSERVO_GETANGLE = 0
DIGITALSERVO_SETANGLE = 1
DIGITALSERVO_SETPOWER = 2

OPTICAL_COLOR = 0
OPTICAL_GRAY_SCALE = 1
OPTICAL_RGB = 2
OPTICAL_AMBIENT_LIGHT_INTENSITY = 3
OPTICAL_KEY_STATUS = 4
OPTICAL_LIGHT_UP = 5
# OPTICAL_SIGNAL_STRENGTH = 6

LASER_DISTANCE_MM = 0
LASER_DISTANCE_CM = 1
LASER_KEY_STATUS = 2
# LASER_SIGNAL_STRENGTH = 3

# FLAME_DIRECTION = 0
# FLAME_INTENSITY = 1
# FLAME_KEY_STATUS = 2
# FLAME_FOOTBALL_DIRECTION = 3

GEOMAGNETIC_FIELD_ANGLE = 0
GEOMAGNETIC_KEY_STATUS = 1

ATTITUDE_DICE = 0
ATTITUDE_KEY_STATUS = 1
ATTITUDE_ACCELERATION_X = 2
ATTITUDE_ACCELERATION_Y = 3
ATTITUDE_ACCELERATION_Z = 4
ATTITUDE_ANGULAR_VELOCITY_X = 5
ATTITUDE_ANGULAR_VELOCITY_Y = 6
ATTITUDE_ANGULAR_VELOCITY_Z = 7
# ATTITUDE_SIGNAL_STRENGTH = 8
# ATTITUDE_INCLINATION_ANGLE_X = 10
# ATTITUDE_STEPS = 9
# ATTITUDE_INCLINATION_ANGLE_Y = 11
# ATTITUDE_INCLINATION_ANGLE_Z = 12

BUZZER_VOLUME = 2
BUZZER_MUSIC = 3

BUZZER_TONE_HIGH = 3
BUZZER_TONE_MIDDLE = 2
BUZZER_TONE_LOW = 1

BUZZER_NOTE_DO = 1
BUZZER_NOTE_RE = 2
BUZZER_NOTE_MI = 3
BUZZER_NOTE_FA = 4
BUZZER_NOTE_SO = 5
BUZZER_NOTE_LA = 6
BUZZER_NOTE_SI = 7




RGBLIGHT_CONTROL = 2

PHOTOSENSITIVE_AMBIENT_LIGHT_INTENSITY = 2

MESH_MODE_MOTOR = 0
MESH_MODE_BALANCE = 1
MESH_MODE_PASSIVE = 2
MESH_MODE_ACTIVE = 3















